﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Pixoneer.NXDL;        // 기본 함수 관련 기능
using Pixoneer.NXDL.NCC;    // 좌표 시스템 관련 기능
using Pixoneer.NXDL.NIO;    // 파일 입출력 관련 기능
using Pixoneer.NXDL.NRS;    // Remote Sensing 관련 기능
using Pixoneer.NXDL.NGR;    // Graphic관련 기능
using Pixoneer.NXDL.NVC;    // Vector 객체와 관련된 기능
using Pixoneer.NXDL.NXMap;  // MapView 관련 기능

namespace XDL_MapView1
{
public partial class Form1 : Form
{
public XRasterIO m_RasterIO;        //영상의 입출력을 담당할 객체 선언
public XVectorIO m_VectorIO;        //벡터 객체의 입출력을 담당할 객체 선언

public Form1()
{
    InitializeComponent();

    String StrError;
    m_RasterIO = new XRasterIO();   // 객체 생성
    if (m_RasterIO.Initialize(out StrError) == false)   // 영상 입출력 객체 초기화
    {
        MessageBox.Show(StrError);
    }

    nxMapLayerComposites1.LayerVisible = true;

    if (!nxMapView1.SetBackgroundMap("C:\\Pixoneer\\XDL1.2\\Config\\XMapBase.xml"))
    {
        MessageBox.Show("배경맵이 설정되지 않았습니다.");
    }

    // 벡터 객체를 생성하고 편집하기 위해 
    nxMapLayerVectorEditor1.Editable = true;

    // 벡터를 저장하기 위해 벡터 IO객체를 생성
    m_VectorIO = new XVectorIO();
    if (m_VectorIO.Initialize(out StrError) == false)
        MessageBox.Show(StrError);
}

private void OpenToolStripMenuItem_Click(object sender, EventArgs e)
{
    // 파일 Open을 수행한다.
    OpenFileDialog openFileDialog = new OpenFileDialog();
    openFileDialog.Filter = "xdm file(*.xdm)|*.xdm||";
    openFileDialog.RestoreDirectory = true;

    if (openFileDialog.ShowDialog() != DialogResult.OK) return;

    string strFilePath = openFileDialog.FileName;

    // FileDriverManger를 통해 XRSLoadFile을 생성한다. 
    // Tiff를 직접로딩하는 경우 영상크기가 큰 경우 도시 속도를 위해서 LOD가 필요하기때문에 필요한 경우 같은 폴더에 .xld파일을 생성한다.
    string strError;
    XRSLoadFile xrsFileInput1 = m_RasterIO.LoadFile(strFilePath, out strError, false, eIOCreateXLDMode.All_NoMsg);
    if (xrsFileInput1 == null) return;

    // 다시 Original Image에 대한 Composite와 Processed Image에 대한 Composite를 생성하여 뷰 컨트롤에 도시한다.
    XDMComposite newComp = CreateComposite(xrsFileInput1, 0, 1, 2);

    // 기존에 생성되어 있는 Composite가 있다면 Lock을 걸어 도시 수행을 잠시 멈춘다. 
    nxMapLayerComposites1.Lock();

    // nxMapLayerComposites1개체에 존재하는 XDMCompManager들을 꺼낸다. XDMCompManager가 실제로 Composite들의 관리를 수행한다. 
    XDMCompManager xdmCompManager = nxMapLayerComposites1.GetXDMCompManager();

    // 생성된 Composite를 XDMCompManager에 추가한다.
    xdmCompManager.AddXDMComposite(ref newComp);

    // 전체 화면 보기를 설정한다.
    nxMapLayerComposites1.ZoomFit();

    // 화면 업데이트를 수행한다. 
    nxMapLayerComposites1.Invalidate();

    // Lock을 다시 풀어 준다. 
    nxMapLayerComposites1.UnLock();
}

XDMComposite CreateComposite(XRSLoadFile xrsFileInput, int nBandIdx0, int nBandIdx1, int nBandIdx2)
{
    // XDMComposite를 생성한다. .
    XDMComposite newComp = new XDMComposite();

    // 로딩된 파일로 부터 밴드의 갯수가 칼라로 그릴것인지 흑백으로 그릴것인지를 판단한다.
    // 만약 3개 미만인 경우는 Gray모드로 그리고, 3개 이상인 경우는 칼라로 그린다.
    int nNumBand = xrsFileInput.NumBand;
    if (nNumBand < 3)   // 3개 미만의 밴드를 가지고 있는 경우
    {
        // Default로 0번째 밴드를 Gray모드로 그린다. 
        XDMBand band = xrsFileInput.GetBandAt(0);

        // Gray Mode로 그리는것을 설정
        newComp.Mode = eCompMode.Gray;

        // Composite의 0번에 Band설정, Gray모드에서는 0번 밴드만을 인식한다. 
        newComp.SetBand(ref band, nBandIdx0);
        // 영상 Enhancement를 위해 Histogram의 범위 설정(정규분포 95% 영역)
        newComp.SetCutType(eCompCutType.Ct95, 0);
        // 영상 Enhancement를 위해 Histogram적용시 전체 영역에 대한 Histogram(Band)설정, Visible은 현재 영역
        newComp.SetStretchCoverage(eCompStretchCoverage.Band, 0);
        // 영상 Enhancement를 위해 Histogram의 Stretch형태 설정(Gaussian으로 설정)
        newComp.SetStretchType(eCompStretchType.Gaussian, 0);

    }
    else // 3개 이상의 밴드를 가지고 있는 경우 
    {
        XDMBand band1 = xrsFileInput.GetBandAt(nBandIdx0);
        XDMBand band2 = xrsFileInput.GetBandAt(nBandIdx1);
        XDMBand band3 = xrsFileInput.GetBandAt(nBandIdx2);
        newComp.Name = xrsFileInput.FileName; // 파일 이름을 Compoiste이름으로 설정

        newComp.Mode = eCompMode.RGB;  // RGB Mode로 그리는것을 설정
        newComp.SetBand(ref band3, nBandIdx0); // Composite의 0번에 Band설정, Blue Channel설정 
        newComp.SetBand(ref band2, nBandIdx1); // Composite의 1번에 Band설정, Green Channel설정
        newComp.SetBand(ref band1, nBandIdx2); // Composite의 2번에 Band설정, Red Channel설정

        for (int i = 0; i < 3; i++)
        {
            // 영상 Enhancement를 위해 Histogram의 범위 설정(정규분포 95% 영역)
            newComp.SetCutType(eCompCutType.Ct95, i);
            // 영상 Enhancement를 위해 Histogram적용시 전체 영역에 대한 Histogram(Band)설정, Visible은 현재 영역
            newComp.SetStretchCoverage(eCompStretchCoverage.Band, i);
            // 영상 Enhancement를 위해 Histogram의 Stretch형태 설정(Gaussian으로 설정)
            newComp.SetStretchType(eCompStretchType.Gaussian, i);
        }
    }

    // 생성된 Composite를 XDMCompManger객체에 추가한다.        
    return newComp;
}

private void showBackgroundToolStripMenuItem_Click(object sender, EventArgs e)
{
    nxMapView1.BackgroundMapVisible = true;
}

private void hideBackgroundToolStripMenuItem_Click(object sender, EventArgs e)
{
    nxMapView1.BackgroundMapVisible = false;
}


private void pointToolStripMenuItem_Click(object sender, EventArgs e)
{
    nxMapLayerVectorEditor1.CreateNewOBJ(Pixoneer.NXDL.NVC.eXvcObjType.Point, null);
}

private void lineToolStripMenuItem_Click(object sender, EventArgs e)
{
    nxMapLayerVectorEditor1.CreateNewOBJ(Pixoneer.NXDL.NVC.eXvcObjType.Line, null);
}

private void ellipseToolStripMenuItem_Click(object sender, EventArgs e)
{
    nxMapLayerVectorEditor1.CreateNewOBJ(Pixoneer.NXDL.NVC.eXvcObjType.Ellipse, null);
}

private void rectangleToolStripMenuItem_Click(object sender, EventArgs e)
{
    nxMapLayerVectorEditor1.CreateNewOBJ(Pixoneer.NXDL.NVC.eXvcObjType.Rectangle, null);
}

private void polyLineToolStripMenuItem_Click(object sender, EventArgs e)
{
    nxMapLayerVectorEditor1.CreateNewOBJ(Pixoneer.NXDL.NVC.eXvcObjType.Polyline, null);
}

private void polygonToolStripMenuItem_Click(object sender, EventArgs e)
{
    nxMapLayerVectorEditor1.CreateNewOBJ(Pixoneer.NXDL.NVC.eXvcObjType.Polygon, null);
}

private void textToolStripMenuItem_Click(object sender, EventArgs e)
{
    nxMapLayerVectorEditor1.CreateNewOBJ(Pixoneer.NXDL.NVC.eXvcObjType.Text, null);
}

private void textBoxToolStripMenuItem_Click(object sender, EventArgs e)
{
    nxMapLayerVectorEditor1.CreateNewOBJ(Pixoneer.NXDL.NVC.eXvcObjType.Textbox, null);
}

private void measurePointToolStripMenuItem_Click(object sender, EventArgs e)
{
    nxMapLayerVectorEditor1.CreateNewOBJ(Pixoneer.NXDL.NVC.eXvcObjType.MeasurePoint, null);
}

private void measureDistanceToolStripMenuItem_Click(object sender, EventArgs e)
{
    nxMapLayerVectorEditor1.CreateNewOBJ(Pixoneer.NXDL.NVC.eXvcObjType.MeasureDist, null);
}

private void measureAreaToolStripMenuItem_Click(object sender, EventArgs e)
{
    nxMapLayerVectorEditor1.CreateNewOBJ(Pixoneer.NXDL.NVC.eXvcObjType.MeasureArea, null);
}

private void measureAngleToolStripMenuItem_Click(object sender, EventArgs e)
{
    nxMapLayerVectorEditor1.CreateNewOBJ(Pixoneer.NXDL.NVC.eXvcObjType.MeasureAngle, null);
}

private void propertyToolStripMenuItem_Click(object sender, EventArgs e)
{
    XvcObj Obj = nxMapLayerVectorEditor1.GetSelectedObj();
    if (Obj == null) return;

    ChangeProperty(Obj);
}
    
public void ChangeProperty(XvcObj Obj)
{
    // 선택된 객체가 null인 경우는 실행하지 않는다.
    if (Obj == null) {   return;  }

    eXvcObjType type = Obj.Type;

    // Property 창을 생성하고 기존 Object의 속성값을 Property창에 설정한다. 
    PropertyXvcObj propertyDlg  = new PropertyXvcObj();
    propertyDlg.m_strObjType    = type.ToString();
    propertyDlg.m_nID           = Obj.ID;
    propertyDlg.m_strName       = Obj.Name;
    propertyDlg.m_crBack        = Obj.ColorBack;
    propertyDlg.m_crFore        = Obj.ColorFore;
    propertyDlg.m_crLine        = Obj.ColorLine;
    propertyDlg.m_dblThick      = Obj.GetThick();
    propertyDlg.m_StyleFill     = Obj.StyleFill;
    propertyDlg.m_StyleLine     = Obj.StyleLine;
    propertyDlg.m_crText        = Obj.ColorText;


    // Object Type에 따라 속성값을 Property창에 설정한다.
    if (type == eXvcObjType.Text)
    {
        propertyDlg.m_dblFontSize   = ((XvcText)Obj).FontSize;
        propertyDlg.m_strFontName   = ((XvcText)Obj).FontName;
        propertyDlg.m_bBold         = ((XvcText)Obj).Bold;
        propertyDlg.m_bUnderline    = ((XvcText)Obj).Underline;
        propertyDlg.m_bItalic       = ((XvcText)Obj).Italic;
        propertyDlg.m_bStrikeOut    = ((XvcText)Obj).StrikeOut;
    }
    else if (type == eXvcObjType.Line)
    {
        propertyDlg.m_LineStartShape = ((XvcLine)Obj).StartShapeType;
        propertyDlg.m_LineEndShape = ((XvcLine)Obj).EndShapeType;
    }


    // Object Type에 따라 속성값을 Property창에 설정한다.
    if (propertyDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
    {
        Obj.ID = propertyDlg.m_nID;
        Obj.Name = propertyDlg.m_strName;
        Obj.ColorBack = propertyDlg.m_crBack;
        Obj.ColorFore = propertyDlg.m_crFore;
        Obj.ColorLine = propertyDlg.m_crLine;
        Obj.SetThick(propertyDlg.m_dblThick);
        Obj.StyleFill = propertyDlg.m_StyleFill;
        Obj.StyleLine = propertyDlg.m_StyleLine;
        Obj.ColorText = propertyDlg.m_crText;

        // Object가 텍스트인경우 Text객체에 Property창에서 정의한 값을 설정한다.
        if (type == eXvcObjType.Text)
        {
            XvcText text = (XvcText)Obj;
            text.FontSize = propertyDlg.m_dblFontSize;
            text.FontName = propertyDlg.m_strFontName;
            text.Bold = propertyDlg.m_bBold;
            text.Underline = propertyDlg.m_bUnderline;
            text.Italic = propertyDlg.m_bItalic;
            text.StrikeOut = propertyDlg.m_bStrikeOut;
        }
        // Object가 Line인경우 Line객체에 Property창에서 정의한 값을 설정한다.
        else if (type == eXvcObjType.Line)
        {
            ((XvcLine)Obj).StartShapeType = propertyDlg.m_LineStartShape;
            ((XvcLine)Obj).EndShapeType = propertyDlg.m_LineEndShape;
        }

        this.nxMapView1.RefreshScreen();
    }
}

// Vector 객체 로딩
private void openXvcVectorToolStripMenuItem_Click(object sender, EventArgs e)
{
    // 열기위한 파일 다이얼로그 창 생성.
    OpenFileDialog openFileDialog = new OpenFileDialog();
    openFileDialog.Multiselect = false;
    openFileDialog.Filter = "XvcBase Files|*.xvml";
    openFileDialog.RestoreDirectory = true;

    if (openFileDialog.ShowDialog() != DialogResult.OK) return;

    // XvcBase객체 생성(XvcObject들을 담기 위한 최상위 객체).
    String strError;
    XvcBase vcBase = new XvcBase();

    // XvcBase객체 로딩
    bool bres = vcBase.LoadFile(openFileDialog.FileName, out strError, null);

    if (bres)
        MessageBox.Show("Vector File Open Succeed", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
    else
    {
        MessageBox.Show("Vector File Open Failed", "Fail", MessageBoxButtons.OK, MessageBoxIcon.Error);
        return;
    }

    // 로딩된 XvcBase객체를 XMapLayerVectorEditor객체에 설정하여 재 편집이 가능하도록 설정
    nxMapLayerVectorEditor1.SetEditBase(vcBase);
}

// 내부 xml 포맷인 XVML파일로 저장
private void pixoneerXVMLToolStripMenuItem_Click(object sender, EventArgs e)
{
    // XvcBase객체 생성(XvcObject들을 담기 위한 최상위 객체).
    Pixoneer.NXDL.NVC.XvcBase vcbase = nxMapLayerVectorEditor1.GetEditBase();
    if (vcbase == null) return;

    // 저장 경로 생성을 위한 Save 파일 다이얼로그 생성
    SaveFileDialog dialog = new SaveFileDialog();
    dialog.Title = "Save XvcBase File";
    dialog.DefaultExt = "xvml";
    dialog.Filter = "XvcBase Files|*.xvml";

    if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
    {
        String strError;

        // XvcBase에 저장된 객체를 파일로 저장
        bool bres = vcbase.SaveFile(dialog.FileName, out strError, null);

        if (bres)
            MessageBox.Show("Vector File Save Succeed", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
        else
            MessageBox.Show("Vector File Save Failed", "Fail", MessageBoxButtons.OK, MessageBoxIcon.Error);
    }
}

// XDL이 지원하는 외부 벡터 객체를 로딩
private void externalVectorFormatToolStripMenuItem_Click(object sender, EventArgs e)
{
    // XMapLayerVectorEditor객체에 있는 XvcBase객체를 꺼내기.
    Pixoneer.NXDL.NVC.XvcBase vcbase = nxMapLayerVectorEditor1.GetEditBase();
    if (vcbase == null) return;

    // XDL이 지원하는 File filter 꺼내기
    String fileFilters = m_VectorIO.GetFiltersForSave();

    // 외부 파일로 저장하기 위해 저장 다이얼로그 생성
    SaveFileDialog dialog = new SaveFileDialog();
    dialog.Filter = fileFilters;

    if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
    {
        String vectorFileName = dialog.FileName;
        String VectorFileKey = ExtractFilterKey(fileFilters, dialog.FilterIndex - 1);

        String strError;

        // 벡터를 외부 파일로 저장.
        bool bres = m_VectorIO.Export(ref vcbase, dialog.FileName, VectorFileKey, out strError, null);

        if (bres)
            MessageBox.Show("Vector File Save Succeed", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
        else
            MessageBox.Show("Vector File Save Failed", "Fail", MessageBoxButtons.OK, MessageBoxIcon.Error);
    }
}

private String ExtractFilterKey(String strFilter, int idx)
{
    int tempidx = 0;
    String strKey = "";
    do
    {
        int idxDelim = strFilter.IndexOf(";|");
        if (idxDelim != -1)
        {
            String strKeyTemp = strFilter.Substring(0, idxDelim).Trim();
            int idxPipe = strKeyTemp.IndexOf('|');
            if (idxPipe != -1)
            {
                strKey = strKeyTemp.Substring(0, idxPipe).Trim();
            }
            else
            {
                strKey = strKeyTemp;
            }

            strFilter = strFilter.Remove(0, idxDelim + 2);
            tempidx++;
        }
    } while (tempidx <= idx);

    return strKey;
}



}
}